/************************************************************************
 **
 **  @file   vpiececarrousellayer.h
 **  @author Ronan Le Tiec
 **  @date   25 4, 2020
 **
 **  @brief
 **  @copyright
 **  This source code is part of the Valentina project, a pattern making
 **  program, whose allow create and modeling patterns of clothing.
 **  Copyright (C) 2020 Valentina project
 **  <https://gitlab.com/smart-pattern/valentina> All Rights Reserved.
 **
 **  Valentina is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  Valentina is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with Valentina.  If not, see <http://www.gnu.org/licenses/>.
 **
 *************************************************************************/

#ifndef VPIECECARROUSELLAYER_H
#define VPIECECARROUSELLAYER_H

#include <QWidget>
#include "vpuzzlelayer.h"
#include "vpiececarrouselpiece.h"

class VPieceCarrousel;

class VPieceCarrouselLayer : public QWidget
{
    Q_OBJECT
public:
    VPieceCarrouselLayer(VPuzzleLayer *layer, VPieceCarrousel *carrousel);
    ~VPieceCarrouselLayer();

    void Init();
    void Refresh();

    /**
     * @brief Clear it clears the carrousel layer from its pieces
     */
    void Clear();

    QList<VPieceCarrouselPiece*> GetCarrouselPieces();

    VPieceCarrousel* GetCarrousel();

    /**
     * @brief GetPuzzleLayer Returns the corresponding VPuzzleLayer
     * @return the VPuzzleLayer
     */
    VPuzzleLayer* GetLayer();

private:
    Q_DISABLE_COPY(VPieceCarrouselLayer)

    VPuzzleLayer *m_layer;
    VPieceCarrousel *m_carrousel;
    QList<VPieceCarrouselPiece*> m_carrouselPieces;

private slots:

    /**
     * @brief on_PieceUpdated This slot is called when a piece was added
     */
    void on_PieceAdded(VPuzzlePiece* piece);

    /**
     * @brief on_PieceUpdated This slot is called when a piece was removed
     */
    void on_PieceRemoved(VPuzzlePiece* piece);

};

#endif // VPIECECARROUSELLAYER_H
